// Beispiel 13.4
/* Plotter */

#include <conio.h>
#include <dos.h>
#include <graphics.h>
#include <stdio.h>
#include <string.h>
#include "b131a.h"

const int plfak_hpgl=400;
  /* Umrechnungsfaktor von cm in absolute
     Plottereinheiten */
const char del=';';
  /* Abschluzeichen eines HPGL-Befehls */

struct Pkt
{ float x,   // x - Koordinate
	y;   // y - Koordinate
};

FILE *pldat;
float pl_xur, pl_yur;

//---- Plotter initialisieren --
// Entspricht initgraph

void plot_on (float xur,float yur,char *plotdatei)
{ // Offset des Koordinatenursprungs
  pl_xur=xur;
  pl_yur=yur;
  // Plotdatei ffnen
  pldat=fopen(plotdatei,"w");
  // Plotter initialisieren

  // Linientyp durchgehend
  // Select Pen Number 1
  // Standardzeichensatz Deutsch
  fprintf(pldat,
    "IN%c\nLT%c\nSP 1%c\nCS 33%c\n",
    del,del,del,del);
} // plot_on

//---- Plot beenden und Plotdatei schlieen
// entspricht closegraph
void plot_end()
{ fprintf(pldat,"SP 0%c\nPU 0,0%c\n",del,del);
  fclose(pldat);
} // plot_end

//---- Koordinatenumrechnung von cm
//---- in absolute Plotterkoordinaten
int pl_x(float x)
{ return (pl_xur+x)*plfak_hpgl;
} // pl_x

int pl_y(float y)
{ return (pl_yur+y)*plfak_hpgl;
} // pl_y

//---- Linie vom aktuellen Punkt
//---- zum angegebenen Endpunkt (lineto)
void pl_liniebis(Pkt endpkt)
{ fprintf(pldat,"PD %d,%d%c\n",
    pl_x(endpkt.x),pl_y(endpkt.y),del);
} // pl_liniebis

//---- Pen positionieren (moveto)
void pl_penposition(Pkt position)
{ fprintf(pldat,"PU %d,%d%c\n",
    pl_x(position.x),pl_y(position.y),del);
} // pl_penposition

//---- Plotten einer Linie (line)
void pl_linie(Pkt startpkt,Pkt endpkt)
{ pl_penposition(startpkt);
  pl_liniebis(endpkt);
} // pl_linie

//---- Kreis zeichnen (circle)
void pl_kreis(Pkt mittelpunkt,float radius)
{ pl_penposition(mittelpunkt);
  fprintf(pldat,"CI %d,2%c\n",
    (int)(radius*plfak_hpgl),del);
} // pl_kreis

//---- Pen auswhlen (set color)
void pl_pen(unsigned pennummer)
{ fprintf(pldat,"SP %u%c\n",pennummer,del);
} // pl_pen

//---- Text plotten
void pl_text(char *ch,float xr,float yr,float zh,float zb)
{ // Richtung der Textzeile
  fprintf(pldat,
    "DI %.4f,%.4f%c\n",xr,yr,del);
  // Zeichenhhe und Breite
  fprintf(pldat,
    "SI %.4f,%.4f%c\n",zb,zh,del);
  fprintf(pldat,"LB %s%c%c\n",ch,'\003',del);
} // pl_text

void main()
{ Pkt pt[6]={{0,3},{2,1},{4,6},
	     {5,3},{8,8},{9,2}};
  Graphik g("..\\bgi");

  // Eine Welt mit der Einheit = cm ffnen
  Fenster f(-2,12,12,-2,20,20,
    f.x_max()-20,f.y_max()-20,LIGHTGRAY,BLUE);

  // Plotter initialisieren
  plot_on(10,10,"DEMO.PLT");

  // Jedes Graphikelement wird zuerst im
  // Fenster und dann am Plotter dargestellt

  // Achsen
  f.achsen(-1,10,10,-1,YELLOW);
  Pkt sp,ep;
  sp.x=-1; sp.y=10;
  ep.x=10; ep.y=-1;
  pl_linie(sp,ep); // x-Achse

  sp.x=0;  sp.y=10;
  ep.x=0;  ep.y=-1;
  pl_linie(sp,ep); // y-Achse

  // Kreis am Startpunkt des Linienzuges
  circle(f.xPix(pt[0].x),f.yPix(pt[0].y),15);
  pl_kreis(pt[0],0.5);

  setcolor(RED);
  moveto(f.xPix(pt[0].x),f.yPix(pt[0].y));
  // Pen 2 an den Anfang des Linienzuges
  pl_pen(2);
  pl_penposition(pt[1]);

  // Linienzug zeichnen
  for (int pktnum=1;pktnum<6;pktnum++)
  { lineto(f.xPix(pt[pktnum].x),
	   f.yPix(pt[pktnum].y));
    pl_liniebis(pt[pktnum]);
  }

  // Pen zum Startpunkt des Textes stellen
  // und dann Text plotten
  moveto(f.xPix(1),f.yPix(10)); // im Fenster
  settextstyle(0,0,3);
  outtext("PLOT-DEMO");
  sp.x=1; sp.y=10; // am Plotter
  pl_penposition(sp);
  pl_text("PLOT-DEMO",1,0,0.8,0.6);
  plot_end();
  f.warte_auf_taste();
  f.bgi_end();
} // b134

